/* -*- mode: c++; c-basic-offset: 4; -*- */
#include <gm_metric.h>
// #include <libmetrics.h>
#include "gds/lsmp.hh"
#include "gds/Time.hh"

extern "C" {

extern mmodule lld_module;

  struct lld_data {
    const char* lld_shm;
    long        lld_gps;
    long        lld_cnt0;
  };

  static lld_data lld_config[] = {
    {"LHO_Data",   0, 0},
    {"LLO_Data",   0, 0},
    {"VIRGO_Data", 0, 0},
    {NULL, 0, 0}
  };

  static long dt = 4;

  static g_val_t
  lld_dropout_rate( int part_index ) {
    g_val_t val;
    val.f = 0;
    LSMP_ACCESS lac(lld_config[part_index].lld_shm);
    if (!lac.attached()) return val;

    /*  Get the current time and number of buffers read.
     */
    long gps = Now().getS();
    long nok = strtol(lac.getStat("buffer_tot").c_str(), 0 ,0);

    /*  Get current values
     */
    long lastgps = lld_config[part_index].lld_gps;
    if (lastgps != 0 && gps != lastgps) {
      val.f = 1.0
	    - float(dt*(nok - lld_config[part_index].lld_cnt0))
	    / float(gps - lastgps);
    }
    lld_config[part_index].lld_gps  = gps;
    lld_config[part_index].lld_cnt0 = nok;
    return val;
  }

static int lld_metric_init ( apr_pool_t *p )
{
    //libmetrics_init();

    for (int i=0; lld_module.metrics_info[i].name != NULL; i++) {
        /* Initialize the metadata storage for each of the metrics and then
         *  store one or more key/value pairs.  The define MGROUPS defines
         *  the key for the grouping attribute. */
        MMETRIC_INIT_METADATA(&(lld_module.metrics_info[i]), p);
        MMETRIC_ADD_METADATA(&(lld_module.metrics_info[i]),
			     MGROUP, "lowlatency");
    }

    return 0;
}

static void lld_metric_cleanup ( void )
{
}

static g_val_t lld_metric_handler ( int metric_index )
{

    /* The metric_index corresponds to the order in which
       the metrics appear in the metric_info and lld_config arrays
    */

    return lld_dropout_rate(metric_index);
}

static Ganglia_25metric lld_metric_info[] = 
{
    {0, (char*) "lld_lho_dropout", 100, 
     GANGLIA_VALUE_FLOAT, (char*) "Hz", (char*) "zero", 
     (char*) "%.0f", UDP_HEADER_SIZE+8, (char*) "LHO data dropout rate"},
    {0, (char*) "lld_llo_dropout", 100, 
     GANGLIA_VALUE_FLOAT, (char*) "Hz", (char*) "zero", 
     (char*) "%.0f", UDP_HEADER_SIZE+8, (char*) "LLO data dropout rate"},
    {0, (char*) "lld_virgo_dropout", 100, 
     GANGLIA_VALUE_FLOAT, (char*) "Hz", (char*) "zero", 
     (char*) "%.0f", UDP_HEADER_SIZE+8, (char*) "VIRGO data dropout rate"},
    {0, 0, 0, GANGLIA_VALUE_FLOAT, 0, 0, 0, 0, 0}
};

mmodule lld_module =
{
    STD_MMODULE_STUFF,
    lld_metric_init,
    lld_metric_cleanup,
    lld_metric_info,
    lld_metric_handler
};

}
